/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <string.h>                         /* strstr() */
#include <stdlib.h>                         /* free() exit() */
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"

#include "initng_no_parallel.h"

/* This is an untested beta module for running a single service at the time for some things, 
    coldplug/static-modules/readahead shouldn't have anything else done at the same time.
    NOTE, we're not 100%, some daemons are marked as DONE/RUNNING very quickly, when in fact they are not done initializing yet.
*/


s_entry NO_PARALLEL = { "no_parallel", SET, 11 };
active_h *target_service;
int target_service_starting;
int extra_check;

static int count_starting_services(void) 
{
    /* Count services in STARTING mode */
    active_h *tmp = g.active_database;
    int num = 0;
    D_("Starting_services, target : %s\n",target_service->name);
    if (target_service->a_status == STARTING)
        return 1;
    while (tmp) {
        if (tmp->a_status == STARTING) {
//            D_("Service %s is STARTING\n",tmp->name);
            num ++;
        }
        tmp=tmp->next;
    }
    D_("Returning %d\n",num);
    return num;
}

/* A LOT of these return TRUE can be replaced with return FALSE and skipping a mark_service over it */
/* Make sure there is only one service starting */
int check_parallel(active_h * service)
{
    if (extra_check)
        return (TRUE);
    if (service->a_status == START_DEP_MET) {
        /* Target_service is set, everything that's starting has to wait */
        if (target_service != NULL) {
            /* If no running services, and target_service not started, start it */
            if (service != target_service || (service == target_service && !target_service_starting)) {
//                D_("Setting WAITING_FOR_START_DEP for service %s\n",service->name);
                mark_service(service, WAITING_FOR_START_DEP);
            }
            if (!target_service_starting && count_starting_services() == 0) {
                D_("Starting service %s alone!\n",target_service->name);
                target_service_starting = TRUE;
                /* This simply runs through the loop again, and in the next loop we wont set our service as WAITING_FOR_START_DEP */
                try_start_wait_deps();
                return (TRUE);
            }
            return (TRUE);
        }
        /* Target service not set, check if we should set it */
        if (active_db_is(&NO_PARALLEL,service)) {
            /* This extra check is needed because of?  use atleast.. I'm not a fan of this system :/ */
            extra_check = TRUE;
            mark_service(service,WAITING_FOR_START_DEP);
            mark_service(service,START_DEP_MET);
            extra_check = FALSE;
            if (service->a_status != START_DEP_MET)
                return (FALSE);
            
            D_("a NO_PARALLEL service found (%s)!\n",service->name);
            target_service = service;
            if (count_starting_services() == 0)
                target_service_starting = TRUE;
            else
                return (FALSE);
        }
        return (TRUE);
    }
    /* Check if target_service is in DONE/RUNNING/FAIL_START, and set it to NULL */
    if (service == target_service && (service->a_status == DONE || service->a_status == RUNNING || service->a_status == FAIL_STARTING)) {
        D_("a NO_PARALLEL service %s has started, everything else is free to start\n",service->name);
        target_service_starting = FALSE;
        target_service = NULL;
    }
    return (TRUE);
}


int module_init(void)
{
    D_("module_init();\n");
    
    target_service_starting = FALSE;
    target_service = NULL;
    extra_check = FALSE;
    
    initng_add_hook(WATCHERS, &check_parallel);
    initng_sdt_add(&NO_PARALLEL);
    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_sdt_del(&NO_PARALLEL);
    initng_del_hook(WATCHERS, &check_parallel);
}
